#!/bin/sh
# This file is part of runit-shutdown which is under the terms of GPL v2+
# Copyright (C) 2005-2006 Alejandro Mery <amery@geeks.cl>

# pidof [-s] [-c] [-x] [-o omitpid] [-o omitpid] program [program..]
# -s only one pid
# -c only on the same root directory
# -x returns the pid of the shells runing scripts named like that
# -o omit pid, %PPID use parent process of `pidof`
#
do_pidof() {
	false
	}

# killall5 -signalnum
#
do_killall5() {
	local my_pid= my_ppid= my_pgrp= my_session=
	local proc_pid= proc_ppid= proc_pgrp= proc_session=
	local signal=

	if [ $# -gt 1 -o "${1:0:1}" != "-" ]; then
		echo "killall5: syntax error"
		echo
	elif [ $# -eq 1 ]; then
		signal="$( echo "$1" | sed -n -e 's,^-\([1-9][0-9]*\)$,\1,p' )"
	fi

	if [ -z "$signal" ]; then
		echo "usage:  killall5 -signalnum"
		exit 2
	elif [ "$signal" -lt 1 -o "$signal" -gt 64 ]; then
		echo "killall5: signal number $signal out of range"
		exit 3
	fi
	
 	# pid (comm) state ppid pgrp session tty_nr tpgid flags ...
	eval `sed -e 's,^\([0-9]*\) ([^)]*) . \([0-9]*\) \([0-9]*\) \([0-9]*\) .*,my_pid=\1 my_ppid=\2 my_pgrp=\3 my_session=\4,g' \
		/proc/$$/stat | tr ' ' '\n'`

	# who am i?
	#echo -e "$my_pid\t$my_ppid\t$my_pgrp\t$my_session\tMYSELF"

	sed -e 's,^\([0-9]*\) ([^)]*) . \([0-9]*\) \([0-9]*\) \([0-9]*\) .*,\1 \2 \3 \4,g' \
		`ls -d1 /proc/*/stat` 2> /dev/null | while read proc_pid proc_ppid proc_pgrp proc_session; do
			case "$proc_session" in
				0|1|$my_session)	action=''	;;
				*)	action="kill -$signal $proc_pid" ;;
			esac

			if [ "$action" ]; then
				# echo -e "$proc_pid\t$proc_pid\t$proc_pgrp\t$proc_session\t$action"
				$action
			fi
		done
	}

case "${0##*/}" in
	killall5)	command="do_killall5" ;;
	#pidof)	command="do_pidof" ;;
	*)	command="" ;;
esac
	
if [ "$command" ]; then
	$command "$@"
	exit $?
fi

echo "${0##*/} not implemented"
exit 1
